/*
 * LOONGARCH gdb server stub
 *
 * Copyright (c) 2003-2005 Fabrice Bellard
 * Copyright (c) 2013 SUSE LINUX Products GmbH
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, see <http://www.gnu.org/licenses/>.
 */
#include "qemu/osdep.h"
#include "qemu-common.h"
#include "cpu.h"
#include "internal.h"
#include "exec/gdbstub.h"
#ifdef CONFIG_TCG
#include "exec/helper-proto.h"
#endif
int loongarch_cpu_gdb_read_register(CPUState *cs, GByteArray *mem_buf, int n)
{
    LOONGARCHCPU *cpu = LOONGARCH_CPU(cs);
    CPULOONGARCHState *env = &cpu->env;

    if (0 <= n && n < 32) {
        return gdb_get_regl(mem_buf, env->active_tc.gpr[n]);
    } else if (n == 32) {
        return gdb_get_regl(mem_buf, env->active_tc.PC);
    }
    return 0;
}

int loongarch_cpu_gdb_write_register(CPUState *cs, uint8_t *mem_buf, int n)
{
    LOONGARCHCPU *cpu = LOONGARCH_CPU(cs);
    CPULOONGARCHState *env = &cpu->env;
    target_ulong tmp = ldtul_p(mem_buf);

    if (0 <= n && n < 32) {
        return env->active_tc.gpr[n] = tmp, sizeof(target_ulong);
    } else if (n == 32) {
        return env->active_tc.PC = tmp, sizeof(target_ulong);
    }
    return 0;
}

static int loongarch_gdb_get_fpu(CPULOONGARCHState *env, GByteArray *mem_buf, int n)
{
    if (0 <= n && n < 32) {
        return gdb_get_reg64(mem_buf, env->active_fpu.fpr[n].d);
    } else if (32 <= n && n < 40) {
        return gdb_get_reg8(mem_buf, env->active_fpu.cf[n - 32]);
    } else if (n == 40) {
        return gdb_get_reg32(mem_buf, env->active_fpu.fcsr0);
    }
    return 0;
}

static int loongarch_gdb_set_fpu(CPULOONGARCHState *env, uint8_t *mem_buf, int n)
{
    if (0 <= n && n < 32) {
        return env->active_fpu.fpr[n].d = ldq_p(mem_buf), 8;
    } else if (32 <= n && n < 40) {
        return env->active_fpu.cf[n - 32] = ldub_p(mem_buf), 1;
    } else if (n == 40) {
        return env->active_fpu.fcsr0 = ldl_p(mem_buf), 4;
    }
    return 0;
}

void loongarch_cpu_register_gdb_regs_for_features(CPUState *cs)
{
    gdb_register_coprocessor(cs, loongarch_gdb_get_fpu, loongarch_gdb_set_fpu,
                              41, "loongarch-fpu64.xml", 0);
}

#ifdef CONFIG_TCG
int loongarch_read_qxfer(CPUState *cs, const char *annex, uint8_t *read_buf,
                         unsigned long offset, unsigned long len)
{
    if (strncmp(annex, "cpucfg", sizeof("cpucfg") - 1) == 0) {
        if (offset % 4 != 0 || len % 4 != 0) {
            return 0;
        }

        size_t i;
        for (i = offset; i < offset + len; i += 4)
            ((uint32_t *)read_buf)[(i - offset) / 4] =
                helper_cpucfg(&(LOONGARCH_CPU(cs)->env), i / 4);
        return 32 * 4;
    }
    return 0;
}

int loongarch_write_qxfer(CPUState *cs, const char *annex,
                          const uint8_t *write_buf, unsigned long offset,
                          unsigned long len)
{
    return 0;
}
#endif
